#!/usr/bin/perl
# Author: Yuichi Nakamura <ynakam@gwu.edu>
# Used some codes from procacl_edit.cgi
# configuration of misc filesystem.
use Fcntl;
use CGI qw(:standard);
require "common.pl";
$Jscript =<<"EOL";
function check_changed(msg) {
    if (parent.config.document.edit_form.elements["changed"].value ==1)
    {
	alert(msg);
    }
}

function set_change()
{
    parent.config.document.edit_form.elements["changed"].value=1;
}

function unset_change() {
    parent.config.document.edit_form.elements["changed"].value=0;
}
EOL

#information of "allowfs" is stored.
#key: filesystem
#value:s|r|w|x separated by space
local %fs_acl;
local %global_fs_acl;


#Information about allowfs tmpfs exclusive.
#key domain name
#value: r|w|s|x separated by space
local %tmpfs_acl;
#1,if domain can create file in tmpfs.
local $tmpfs_create_flag;
local $domain;
local $Policy_root = set_policy_root();

#key: s|r|x|w
#value: 1|0 (if allowpriv search|read|write|all is set then 1
local %all_flag;

$domain=param("domain");
if ($domain eq ""){
    &print_header();
    print "<html> </html>";    
    return;
}


$all_flag{"r"}= check_adm($domain, $Policy_root, "read");
$all_flag{"s"} = check_adm($domain, $Policy_root, "search");
$all_flag{"x"} = check_adm($domain, $Policy_root, "execute");
$all_flag{"w"}= check_adm($domain, $Policy_root, "write");

&read_fs_acl($domain,\%fs_acl);
if ($domain ne "global"){
    &read_fs_acl("global",\%global_fs_acl);
}



&print_table();

#Domain names which can create file in tmpfs
#key:domainname ,value:1
local %Domain_list;
&read_tmpfs_create_list(\%Domain_list,$Policy_root);
&read_tmpfs_acl($domain,\%tmpfs_acl,\$tmpfs_create_flag);
if($domain ne "global"){
    &print_tmpfs_table();
}
print submit(-value=>$text{"apply"},-onClick=>"unset_change()");
print end_form();
print end_html();

sub print_table() {
    &print_header();
    my @supported_fs =@SUPPORTED_FS;

    my $notchange=$text{"notchanged"};
    print start_html(-title=>$text{"fs_acl_title"},-onUnload=>"check_changed(\'$notchange\')");
    print "<SCRIPT language=\"Javascript\">";
    print $Jscript;
    print "</SCRIPT>";

    print h2(&text("fs_acl_title",$domain));

    print start_form(-method=>"post",-action=>"setfsacl.cgi",-name=>"edit_form");
    print hidden(-name=>"domain",-value=>"$domain");
    print hidden(-name=>"changed",-value=>"0");

    my @table;
    push @table,td(["","$text{stat}","$text{read}","$text{exec}","$text{write}"]);
    
    my $fs;
    foreach $fs (@supported_fs){
	&make_field($fs, $text{"fs_$fs"}, \@table);
    }
#    push @table,td([submit(-value=>$text{"apply"},-onClick=>"unset_change()")]);

    print table(Tr(\@table));
 #   print end_form(),end_html();
}

sub make_one_checkstr($$){
    my $key = $_[0];
    my $perm = $_[1];
    my $checkstr;

    if ($global_fs_acl{$key}=~/$perm/ || $all_flag{$perm} == 1){
	$checkstr=$text{"ok"};
    }else{
	my $checked=0;
	if ($fs_acl{$key}=~/$perm/){
	    $checked=1;
	}
	$checkstr=checkbox(-name=>"$key"."_$perm",-label=>"",-value=>"on",-checked=>"$checked",-onChange=>"set_change()");
    }    
    return $checkstr;
}

#construct row
sub make_field($$$) {
    my $key;
    my $menustr;
    my $table_ref;
    ($key,$menustr,$table_ref)=@_;

    my $checksstr = make_one_checkstr($key,"s");
    my $checkrstr = make_one_checkstr($key,"r");
    my $checkxstr = make_one_checkstr($key,"x");
    my $checkwstr = make_one_checkstr($key,"w");

    push @$table_ref,td(["$menustr",$checksstr,$checkrstr,$checkxstr,$checkwstr]);
}

sub read_fs_acl($$) {
    my $domain;
    my $acl_hash_ref;
    my $filename;
    
    ($domain,$acl_hash_ref)=@_;
    $filename = get_policy_file_name($domain, $Policy_root);

    sysopen(FILE,"$filename",O_RDONLY) or die "File open error:$filename\n";

    while (<FILE>){
	#skip space in the head of line
	if (/[^\s\t]+.+/){
	    $_=$&;
	}
	#allow <filesystem> <permissions>;
	if (/^allowfs[\s\t]+\S+[\s\t]+[\w,^;]+[\s\t]*;/){
	    my @tmp;
	    my $opt;
	    my $perm;
	    @tmp= split(/[\s\t]+/,$_);
	    $opt=$tmp[1];
	    $perm=$tmp[2];
	    if ($perm=~/;/){
		chop($perm);
	    }
	    my @tmp2;
	    @tmp2=split(/,/,$perm);

	    $$acl_hash_ref{$opt}.=" "."@tmp2";
	}	
    }    
    close(FILE);
}

sub print_tmpfs_table() {

    print h2(&text("tmpfs_exclusive"));

    my $checkstr;

    print start_form(-method=>"post",-action=>"settmpfsacl.cgi",-name=>"edit_form");
    print hidden(-name=>"domain",-value=>"$domain");
    print hidden(-name=>"changed",-value=>"0");


    if ($tmpfs_create_flag==1){
	$checked=1;
    }
    $checkstr=checkbox(-name=>"tmpfsexc_create",-label=>"",-value=>"on",-checked=>"$checked",-onChange=>"set_change()");
    
    if ($domain ne "global"){
	print $text{"tmpfs_create"},$checkstr,p;
    }
    my @table;

    print $text{"tmpfs_list"};
    delete $Domain_list{"global"};
    delete $Domain_list{"general"};
    my  @domain_array=keys %Domain_list;
    @domain_array=sort @domain_array;
    my $d;
    foreach $d (@domain_array){
	&make_tmpfs_field(\@table, $d, \%tmpfs_acl); 
    }
    make_tmpfs_field(\@table,"-all", \%tmpfs_acl);
    push @table,td(["",$text{"stat"},$text{"read"},$text{"exec"},$text{"write"}]);

    print table(Tr(\@table));

}


sub make_one_tmpfs_checkstr($$){
    my $key = $_[0];
    my $perm = $_[1];
    my $checkstr;

    my $checked=0;
    if ($tmpfs_acl{$key}=~/$perm/){
	$checked=1;
    }
    
    if ($all_flag{$perm} == 1){
	$checkstr=$text{"ok"};
    }else{
	$checkstr=checkbox(-name=>"tmpfsexc/$key"."/$perm",-label=>"",-value=>"on",-checked=>"$checked",-onChange=>"set_change()");
    }
    return $checkstr;
}

sub make_tmpfs_field($$){
    my $table_ref;
    my $domainname;
    my $leftstr;
    ($table_ref,$domainname)=@_;
    
    my $checksstr = make_one_tmpfs_checkstr($domainname,"s");
    my $checkrstr = make_one_tmpfs_checkstr($domainname,"r");
    my $checkxstr = make_one_tmpfs_checkstr($domainname,"x");
    my $checkwstr = make_one_tmpfs_checkstr($domainname,"w");
    
    if($domainname eq "-all"){
	$leftstr=$text{"all"};
    }else{
	$leftstr=$domainname;
    }
	push @$table_ref,td(["$leftstr",$checksstr,$checkrstr,$checkxstr,$checkwstr]);

}

sub read_tmpfs_acl($$$) {
    my $domain;
    my $hash_ref;
    my $filename;
    my $create_flag_ref;

    ($domain,$hash_ref,$create_flag_ref)=@_;

    $filename = get_policy_file_name($domain, $Policy_root);

    sysopen(FILE,"$filename",O_RDONLY) or die "File open error:$filename\n";
    while (<FILE>){

	#skip spaces
	if (/[^\s\t]+.+/){
	    $_=$&;
	}
	#allowfs tmpfs exclusive <label>;
	if(/^allowfs[\s\t]+tmpfs[\s\t]+exclusive[\s\t]+[\w_]+[\s\t]*;/){
	    $$create_flag_ref=1;
	    next;
	}	
	
	my $label;
	my $permstr;
	#allowfs tmpfs <label> <permissions>;
	if (/^allowfs[\s\t]+tmpfs[\s\t]+\S+[\s\t]+[\w,^;]+[\s\t]*;/){
	    my @tmp;
	    @tmp= split(/[\s\t]+/,$_);
	    $label = $tmp[2];   
	    $permstr=$tmp[3];
	    if ($permstr=~/;/){
		chop($permstr);
	    }	 
	}

	#allowfs tmpfs excluxive -all <permissions>;
	if(/^allowfs[\s\t]+tmpfs[\s\t]+exclusive[\s\t]+-all/){
	    $label = "-all";
	    my @tmp;
	    @tmp= split(/[\s\t]+/,$_);
	    $permstr=$tmp[4];
	    if ($permstr=~/;/){
		chop($permstr);
	    }
	}

	if($label ne ""){
	    if($label=~/^[a-z0-9]+/){
		$label=$&."_t";
	    }
	}
	if ($permstr=~/r/){
	    $$hash_ref{$label}.=" r";
	}
	if ($permstr=~/w/){
	    $$hash_ref{$label}.=" w";
	}
	if ($permstr=~/x/){
	    $$hash_ref{$label}.=" x";
	}
	if ($permstr=~/s/){
	    $$hash_ref{$label}.=" s";
	}    
    }
    close(FILE);
}

sub read_tmpfs_create_list() {
    my $ref;
    my $path;
    my $filename;
    ($ref,$path)=@_;

    opendir(DIR,$path) or die  "could not open $path";
    while ($_=readdir(DIR)){
	if (/\.a$/){
	    $filename=$path."/".$_;
	}
	elsif ($_ eq "global"){
	    $filename=$path."/".$_;
	}else{
	    next;
	}
	
	sysopen(FILE,"$filename",O_RDONLY) or die "File open error:$filename\n";
	my $dname;

	while(<FILE>){
	    my @s;
	    #skip spaces
	    if (/[^\s\t]+.+/){
		$_=$&;
	    }

	    #skip comments
	    if ( /[\s\t]+\#/||/^\#/){
		next;
	    }

	    if (/^domain[\s\t]/){
		@s=split(/[\s\t]+/,$_);
		$dname=$s[1];
		if ($dname=~/;/){
		    chop($dname);
		}
	    }

	    if (/^role[\s\t]/){
		@s=split(/[\s\t]+/,$_);
		$dname=$s[1];
		if ($dname=~/;/){
		    chop($dname);
		}
	    }

	    #allowfs tmpfs exclusive -all <permissions>;
	    if(/^allowfs[\s\t]+tmpfs[\s\t]+exclusive[\s\t]+-all/){
		next;
	    }

	    #allowfs tmpfs exclusive <label>;
	    if (/^allowfs[\s\t]+[\S]+[\s\t]+exclusive/){
		$$ref{$dname}=1;
		last;
	    }
	}
	close(FILE);
    }
    close(DIR);
}
